@@ -8,7 +8,9 @@
  */
 
 /*
- * clipboard.c: Functions to handle the clipboard
+ * clipboard.c: Functions to handle the clipboard. Additionally contains the
+ *		clipboard provider code, which is separate from the main
+ *		clipboard code.
  */
 
 #include "vim.h"
@@ -29,7 +31,12 @@
 // versions of these for the 'clipboard' selection, as Visual mode has no use
 // for them.
 
-#if defined(FEAT_CLIPBOARD) || defined(PROTO)
+
+#ifdef FEAT_CLIPBOARD_PROVIDER
+static int clip_provider_is_available(char_u *provider);
+#endif
+
+#if defined(FEAT_CLIPBOARD)
 
 #if defined(FEAT_WAYLAND_CLIPBOARD)
 
@@ -100,10 +107,6 @@ typedef struct {
 
     clip_wl_selection_T regular;
     clip_wl_selection_T primary;
-
-    // Array of file descriptors of clients we are sending data to. These should
-    // be polled for POLLOUT and have the respective callback called for each.
-    garray_T write_fds;
 } clip_wl_T;
 
 // Mime types we support sending and receiving
@@ -128,7 +131,7 @@ static int clip_wl_own_selection(Clipboard_T *cbd);
 static void clip_wl_lose_selection(Clipboard_T *cbd);
 static void clip_wl_set_selection(Clipboard_T *cbd);
 
-# if defined(USE_SYSTEM) || defined(PROTO)
+# if defined(USE_SYSTEM)
 static bool clip_wl_owner_exists(Clipboard_T *cbd);
 # endif
 
@@ -172,22 +175,6 @@ skip:
     }
 }
 
-    static void
-clip_init_single(Clipboard_T *cb, int can_use)
-{
-    // No need to init again if cbd is already available
-    if (can_use && cb->available)
-	return;
-
-    cb->available  = can_use;
-    cb->owned      = FALSE;
-    cb->start.lnum = 0;
-    cb->start.col  = 0;
-    cb->end.lnum   = 0;
-    cb->end.col    = 0;
-    cb->state      = SELECT_CLEARED;
-}
-
 /*
  * Check whether the VIsual area has changed, and if so try to become the owner
  * of the selection, and free any old converted selection we may still have
@@ -231,34 +218,32 @@ clip_update_selection(Clipboard_T *clip)
 }
 
     static int
-clip_gen_own_selection(Clipboard_T *cbd UNUSED)
+clip_gen_own_selection(Clipboard_T *cbd)
 {
-    if (clipmethod == CLIPMETHOD_GUI)
-    {
-#ifdef FEAT_GUI
-	if (gui.in_use)
-	    return clip_mch_own_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
-    {
-#ifdef FEAT_WAYLAND_CLIPBOARD
-	return clip_wl_own_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
-#ifdef FEAT_XCLIPBOARD
-	return clip_xterm_own_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_OTHER)
-    {
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
+# ifdef FEAT_GUI
+    if (gui.in_use)
 	return clip_mch_own_selection(cbd);
+    else
+# endif
+    {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
+#ifdef FEAT_WAYLAND_CLIPBOARD
+	    return clip_wl_own_selection(cbd);
 #endif
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
+#ifdef FEAT_XCLIPBOARD
+	    return clip_xterm_own_selection(cbd);
+#endif
+	}
     }
     return FAIL;
+#else
+    return clip_mch_own_selection(cbd);
+#endif
 }
 
     void
@@ -298,33 +283,31 @@ clip_own_selection(Clipboard_T *cbd)
 }
 
     static void
-clip_gen_lose_selection(Clipboard_T *cbd UNUSED)
+clip_gen_lose_selection(Clipboard_T *cbd)
 {
-    if (clipmethod == CLIPMETHOD_GUI)
-    {
-#ifdef FEAT_GUI
-	if (gui.in_use)
-	    clip_mch_lose_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
-    {
-#ifdef FEAT_WAYLAND_CLIPBOARD
-	clip_wl_lose_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
-#ifdef FEAT_XCLIPBOARD
-	clip_xterm_lose_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_OTHER)
-    {
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
+# ifdef FEAT_GUI
+    if (gui.in_use)
 	clip_mch_lose_selection(cbd);
+    else
+# endif
+    {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
+#ifdef FEAT_WAYLAND_CLIPBOARD
+	    clip_wl_lose_selection(cbd);
 #endif
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
+#ifdef FEAT_XCLIPBOARD
+	    clip_xterm_lose_selection(cbd);
+#endif
+	}
     }
+#else
+    clip_mch_lose_selection(cbd);
+#endif
 }
 
     void
@@ -1053,7 +1036,7 @@ clip_process_selection(
 #endif
 }
 
-# if defined(FEAT_GUI) || defined(PROTO)
+# if defined(FEAT_GUI)
 /*
  * Redraw part of the selection if character at "row,col" is inside of it.
  * Only used for the GUI.
@@ -1350,65 +1333,61 @@ clip_gen_set_selection(Clipboard_T *cbd)
 	    return;
 	}
     }
-    if (clipmethod == CLIPMETHOD_GUI)
-    {
-#ifdef FEAT_GUI
-	if (gui.in_use)
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
+# ifdef FEAT_GUI
+    if (gui.in_use)
 	clip_mch_set_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
+    else
+# endif
     {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
 #ifdef FEAT_WAYLAND_CLIPBOARD
-	clip_wl_set_selection(cbd);
+	    clip_wl_set_selection(cbd);
 #endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
 #ifdef FEAT_XCLIPBOARD
-	clip_xterm_set_selection(cbd);
+	    clip_xterm_set_selection(cbd);
 #endif
+	}
     }
-    else if (clipmethod == CLIPMETHOD_OTHER)
-    {
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
-	clip_mch_set_selection(cbd);
+#else
+    clip_mch_set_selection(cbd);
 #endif
-    }
 }
 
     static void
-clip_gen_request_selection(Clipboard_T *cbd UNUSED)
+clip_gen_request_selection(Clipboard_T *cbd)
 {
-    if (clipmethod == CLIPMETHOD_GUI)
-    {
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
 # ifdef FEAT_GUI
-	if (gui.in_use)
-	    clip_mch_request_selection(cbd);
-# endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
-    {
-#ifdef FEAT_WAYLAND_CLIPBOARD
-	clip_wl_request_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
-#ifdef FEAT_XCLIPBOARD
-	clip_xterm_request_selection(cbd);
-#endif
-    }
-    else if (clipmethod == CLIPMETHOD_OTHER)
-    {
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
+    if (gui.in_use)
 	clip_mch_request_selection(cbd);
+    else
+# endif
+    {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
+#ifdef FEAT_WAYLAND_CLIPBOARD
+	    clip_wl_request_selection(cbd);
 #endif
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
+#ifdef FEAT_XCLIPBOARD
+	    clip_xterm_request_selection(cbd);
+#endif
+	}
     }
+#else
+    clip_mch_request_selection(cbd);
+#endif
 }
 
 #if (defined(FEAT_X11) && defined(FEAT_XCLIPBOARD) && defined(USE_SYSTEM)) \
-	|| defined(PROTO)
+
     static int
 clip_x11_owner_exists(Clipboard_T *cbd)
 {
@@ -1416,33 +1395,36 @@ clip_x11_owner_exists(Clipboard_T *cbd)
 }
 #endif
 
-#if ((defined(FEAT_X11) || defined(FEAT_WAYLAND_CLIPBOARD)) \
-	&& defined(USE_SYSTEM)) || defined(PROTO)
+#if (defined(FEAT_X11) || defined(FEAT_WAYLAND_CLIPBOARD)) \
+	&& defined(USE_SYSTEM)
     int
 clip_gen_owner_exists(Clipboard_T *cbd UNUSED)
 {
-    if (clipmethod == CLIPMETHOD_OTHER)
-    {
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
 # ifdef FEAT_GUI_GTK
-	if (gui.in_use)
-	    return clip_gtk_owner_exists(cbd);
-# endif
-    }
-    else if (clipmethod == CLIPMETHOD_WAYLAND)
-    {
-# ifdef FEAT_WAYLAND_CLIPBOARD
-	return clip_wl_owner_exists(cbd);
-# endif
-    }
-    else if (clipmethod == CLIPMETHOD_X11)
-    {
-# ifdef FEAT_XCLIPBOARD
-	return clip_x11_owner_exists(cbd);
-# endif
-    }
+    if (gui.in_use)
+	return clip_gtk_owner_exists(cbd);
     else
-	return FALSE;
-    return FALSE;
+# endif
+    {
+	if (clipmethod == CLIPMETHOD_WAYLAND)
+	{
+#ifdef FEAT_WAYLAND_CLIPBOARD
+	    return clip_wl_owner_exists(cbd);
+#endif
+	}
+	else if (clipmethod == CLIPMETHOD_X11)
+	{
+#ifdef FEAT_XCLIPBOARD
+	    return clip_x11_owner_exists(cbd);
+#endif
+	}
+	else
+	    return FALSE;
+    }
+#else
+    return TRUE;
+#endif
 }
 #endif
 
@@ -1548,7 +1530,7 @@ did_set_clipboard(optset_T *args UNUSED)
  * Stuff for the X clipboard.  Shared between VMS and Unix.
  */
 
-#if defined(FEAT_XCLIPBOARD) || defined(FEAT_GUI_X11) || defined(PROTO)
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_GUI_X11)
 # include <X11/Xatom.h>
 # include <X11/Intrinsic.h>
 
@@ -1740,7 +1722,7 @@ clip_x11_notify_cb(Widget w UNUSED, Atom *sel_atom UNUSED, Atom *target UNUSED)
 /*
  * Property callback to get a timestamp for XtOwnSelection.
  */
-# if (defined(FEAT_X11) && defined(FEAT_XCLIPBOARD)) || defined(PROTO)
+# if defined(FEAT_X11) && defined(FEAT_XCLIPBOARD)
     static void
 clip_x11_timestamp_cb(
     Widget	w,
@@ -2035,8 +2017,7 @@ clip_x11_set_selection(Clipboard_T *cbd UNUSED)
 
 #endif
 
-#if defined(FEAT_XCLIPBOARD) || defined(FEAT_GUI_X11) \
-    || defined(FEAT_GUI_GTK) || defined(PROTO)
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_GUI_X11) || defined(FEAT_GUI_GTK)
 /*
  * Get the contents of the X CUT_BUFFER0 and put it in "cbd".
  */
@@ -2112,7 +2093,7 @@ yank_cut_buffer0(Display *dpy, Clipboard_T *cbd)
  * 'permanent' of the two), otherwise the PRIMARY one.
  * For now, use a hard-coded sanity limit of 1Mb of data.
  */
-#if (defined(FEAT_X11) && defined(FEAT_CLIPBOARD)) || defined(PROTO)
+#if defined(FEAT_X11) && defined(FEAT_CLIPBOARD)
     void
 x11_export_final_selection(void)
 {
@@ -2434,7 +2415,7 @@ adjust_clip_reg(int *rp)
     if ((!clip_star.available && *rp == '*') ||
 	   (!clip_plus.available && *rp == '+'))
     {
-	msg_warn_missing_clipboard(!clip_plus.available, !clip_star.available);
+	msg_warn_missing_clipboard();
 	*rp = 0;
     }
 }
@@ -2880,6 +2861,8 @@ clip_init_wayland(void)
     if (wayland_ct == NULL)
 	return FAIL;
 
+    memset(&clip_wl, 0, sizeof(clip_wl));
+
     clip_wl.seat = vwl_connection_get_seat(wayland_ct, (char *)p_wse);
 
     if (clip_wl.seat == NULL)
@@ -2897,12 +2880,14 @@ clip_init_wayland(void)
 	    clip_wl.regular.available = true;
 	else
 	{
+	    // Shouldn't happen
 	    vwl_data_device_manager_discard(clip_wl.regular.manager);
 	    clip_wl.regular.manager = NULL;
+	    return FAIL;
 	}
     }
 
-    // If we still don't support the primary selection, find one for it
+    // If we still don't support the primary selection, try finding one for it
     // specifically.
     if (!(supported & WAYLAND_SELECTION_PRIMARY))
     {
@@ -2923,10 +2908,12 @@ clip_init_wayland(void)
 	    }
 	}
     }
-    else if (clip_wl.regular.available)
+
+    if (clip_wl.regular.available && !clip_wl.primary.available)
     {
 	// The protocol supports both regular and primary selections, just use
-	// one data device manager and one data device.
+	// one data device manager and one data device. Or the primary selection
+	// is not supported, make it point to the regular selection instead.
 	clip_wl.primary.available = true;
 	clip_wl.primary.manager = clip_wl.regular.manager;
 	clip_wl.primary.device = clip_wl.regular.device;
@@ -3416,7 +3403,7 @@ clip_wl_set_selection(Clipboard_T *cbd UNUSED)
 {
 }
 
-#if defined(USE_SYSTEM) || defined(PROTO)
+#if defined(USE_SYSTEM)
 /*
  * Return true if we own the selection corresponding to cbd or another client
  * does.
@@ -3435,13 +3422,16 @@ clip_wl_owner_exists(Clipboard_T *cbd)
 
 #endif // FEAT_WAYLAND_CLIPBOARD
 
+#endif // FEAT_CLIPBOARD
+
+#ifdef HAVE_CLIPMETHOD
 
 /*
  * Returns the first method for accessing the clipboard that is available/works,
  * depending on the order of values in str.
  */
     static clipmethod_T
-get_clipmethod(char_u *str, bool *regular, bool *primary)
+get_clipmethod(char_u *str)
 {
     int		len	= (int)STRLEN(str) + 1;
     char_u	*buf	= alloc(len);
@@ -3466,11 +3456,7 @@ get_clipmethod(char_u *str, bool *regular, bool *primary)
 	    {
 #ifdef FEAT_WAYLAND_CLIPBOARD
 		if (clip_wl.regular.available || clip_wl.primary.available)
-		{
 		    method = CLIPMETHOD_WAYLAND;
-		    *regular = clip_wl.regular.available;
-		    *primary = clip_wl.primary.available;
-		}
 #endif
 	    }
 	}
@@ -3481,42 +3467,46 @@ get_clipmethod(char_u *str, bool *regular, bool *primary)
 #endif
 	    {
 #ifdef FEAT_XCLIPBOARD
-		// x_IOerror_handler() in os_unix.c should set xterm_dpy to NULL
-		// if we lost connection to the X server.
+		// x_IOerror_handler() in os_unix.c should set xterm_dpy to NULL if
+		// we lost connection to the X server.
 		if (xterm_dpy != NULL)
 		{
-		    // If the X connection is lost then that handler will
-		    // longjmp somewhere else, in that case we will call
-		    // choose_clipmethod() again from there, and this if block
-		    // won't be executed since xterm_dpy will be set to NULL.
+		    // If the X connection is lost then that handler will longjmp
+		    // somewhere else, in that case we will call choose_clipmethod()
+		    // again from there, and this if block won't be executed since
+		    // xterm_dpy will be set to NULL.
 		    xterm_update();
 		    method = CLIPMETHOD_X11;
-		    *regular = *primary = true;
 		}
 #endif
 	    }
 	}
-	else if (STRCMP(buf, "gui") == 0)
-	{
-#ifdef FEAT_GUI
-	    if (gui.in_use)
-	    {
-		method = CLIPMETHOD_GUI;
-		*regular = *primary = true;
-	    }
-#endif
-	}
-	else if (STRCMP(buf, "other") == 0)
-	{
-#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
-		method = CLIPMETHOD_OTHER;
-		*regular = *primary = true;
-#endif
-	}
 	else
 	{
-	    ret = CLIPMETHOD_FAIL;
-	    goto exit;
+#ifdef FEAT_EVAL
+	    // Check if name matches a clipboard provider
+	    int r = clip_provider_is_available(buf);
+
+	    if (r == 1)
+	    {
+		method = CLIPMETHOD_PROVIDER;
+		if (ret == CLIPMETHOD_FAIL)
+		{
+		    vim_free(clip_provider);
+		    clip_provider = vim_strsave(buf);
+		    if (clip_provider == NULL)
+			goto fail;
+		}
+	    }
+	    else if (r == -1)
+#endif
+	    {
+#ifdef FEAT_EVAL
+fail:
+#endif
+		ret = CLIPMETHOD_FAIL;
+		goto exit;
+	    }
 	}
 
 	// Keep on going in order to catch errors
@@ -3545,10 +3535,10 @@ clipmethod_to_str(clipmethod_T method)
 	    return (char_u *)"wayland";
 	case CLIPMETHOD_X11:
 	    return (char_u *)"x11";
-	case CLIPMETHOD_GUI:
-	    return (char_u *)"gui";
-	case CLIPMETHOD_OTHER:
-	    return (char_u *)"other";
+	case CLIPMETHOD_PROVIDER:
+#ifdef FEAT_EVAL
+	    return clip_provider;
+#endif
 	default:
 	    return (char_u *)"none";
     }
@@ -3561,45 +3551,81 @@ clipmethod_to_str(clipmethod_T method)
     char *
 choose_clipmethod(void)
 {
-    bool regular = false, primary = false;
-    clipmethod_T method = get_clipmethod(p_cpm, &regular, &primary);
+    clipmethod_T method = get_clipmethod(p_cpm);
 
     if (method == CLIPMETHOD_FAIL)
 	return e_invalid_argument;
 
-#if defined(FEAT_GUI) && defined(FEAT_WAYLAND)
-    if (method == CLIPMETHOD_GUI)
+// If GUI is running or we are not on a system with Wayland or X11, then always
+// return CLIPMETHOD_NONE. System or GUI clipboard handling always overrides.
+// This is unless a provider is being used.
+#if defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)
+# if defined(FEAT_GUI)
+    if (method != CLIPMETHOD_PROVIDER && gui.in_use)
+    {
+#  ifdef FEAT_WAYLAND
 	// We only interact with Wayland for the clipboard, we can just deinit
 	// everything.
 	wayland_uninit_connection();
+#  endif
+
+	method = CLIPMETHOD_NONE;
+	goto lose_sel_exit;
+    }
+# endif
+#else
+    // If on a system like windows or macos, then clipmethod is irrelevant, we
+    // use their way of accessing the clipboard. This is unless we are using the
+    // clipboard provider
+#ifdef FEAT_CLIPBOARD_PROVIDER
+    if (method != CLIPMETHOD_PROVIDER)
+#endif
+    {
+	method = CLIPMETHOD_NONE;
+	goto exit;
+    }
 #endif
 
+#ifdef FEAT_CLIPBOARD
     // Deinitialize clipboard if there is no way to access clipboard
     if (method == CLIPMETHOD_NONE)
 	clip_init(FALSE);
     // If we have a clipmethod that works now, then initialize clipboard
     else if (clipmethod == CLIPMETHOD_NONE && method != CLIPMETHOD_NONE)
     {
-	clip_init_single(&clip_plus, regular);
-	clip_init_single(&clip_star, primary);
-	clip_plus.did_warn = false;
-	clip_star.did_warn = false;
+	clip_init(TRUE);
+	did_warn_clipboard = false;
     }
     // Disown clipboard if we are switching to a new method
     else if (clipmethod != CLIPMETHOD_NONE && method != clipmethod)
     {
+#if (defined(FEAT_XCLIPBOARD) || defined(FEAT_WAYLAND_CLIPBOARD)) \
+	&& defined(FEAT_GUI)
+lose_sel_exit:
+#endif
 	if (clip_star.owned)
 	    clip_lose_selection(&clip_star);
 	if (clip_plus.owned)
 	    clip_lose_selection(&clip_plus);
-	clip_init_single(&clip_plus, regular);
-	clip_init_single(&clip_star, primary);
+
+#if defined(FEAT_GUI)
+	if (!gui.in_use)
+#endif
+	{
+	    clip_init(TRUE);
+	    did_warn_clipboard = false;
+	}
     }
+#endif // FEAT_CLIPBOARD
+
+#if !defined(FEAT_XCLIPBOARD) && !defined(FEAT_WAYLAND_CLIPBOARD)
+exit:
+#endif
 
     clipmethod = method;
 
 #ifdef FEAT_EVAL
-    set_vim_var_string(VV_CLIPMETHOD, clipmethod_to_str(method), -1);
+    set_vim_var_string(VV_CLIPMETHOD, (char_u*)clipmethod_to_str(method), -1);
 #endif
 
     return NULL;
@@ -3622,4 +3648,407 @@ ex_clipreset(exarg_T *eap UNUSED)
 		clipmethod_to_str(clipmethod));
 }
 
-#endif // FEAT_CLIPBOARD
+#endif // HAVE_CLIPMETHOD
+
+#ifdef FEAT_CLIPBOARD_PROVIDER
+
+/*
+ * Check if a clipboard provider with given name is available. Returns 1 if available,
+ * 0 if not available, and -1 on error
+ */
+    static int
+clip_provider_is_available(char_u *provider)
+{
+    dict_T	*providers = get_vim_var_dict(VV_CLIPPROVIDERS);
+    typval_T	provider_tv = {0};
+    callback_T	callback = {0};
+    typval_T	rettv = {0};
+    typval_T	func_tv = {0};
+    int		res = 0;
+
+    if (dict_get_tv(providers, (char *)provider, &provider_tv) == FAIL
+	    || provider_tv.v_type != VAR_DICT)
+	// clipboard provider not defined
+	return -1;
+
+    if (dict_get_tv(provider_tv.vval.v_dict, "available", &func_tv) == FAIL)
+    {
+	clear_tv(&provider_tv);
+	// If "available" function not specified assume always TRUE
+	return 1;
+    }
+
+    if ((callback = get_callback(&func_tv)).cb_name == NULL)
+	goto fail;
+
+    if (call_callback(&callback, -1, &rettv, 0, NULL) == FAIL ||
+	    (rettv.v_type != VAR_BOOL && rettv.v_type != VAR_NUMBER))
+	goto fail;
+
+    if (rettv.vval.v_number)
+	res = 1;
+
+    if (FALSE)
+fail:
+	res = -1;
+
+    free_callback(&callback);
+    clear_tv(&func_tv);
+    clear_tv(&rettv);
+    clear_tv(&provider_tv);
+
+    return res;
+}
+
+/*
+ * Get the specified callback "function" from the provider dictionary for
+ * register "reg".
+ */
+    static int
+clip_provider_get_callback(
+	char_u *reg,
+	char_u *provider,
+	char_u *function,
+	callback_T *callback)
+{
+    dict_T	*providers = get_vim_var_dict(VV_CLIPPROVIDERS);
+    typval_T	provider_tv;
+    typval_T	action_tv;
+    typval_T	func_tv;
+    callback_T	cb;
+
+    if (dict_get_tv(providers, (char *)provider, &provider_tv) == FAIL)
+	return FAIL;
+    else if (provider_tv.v_type != VAR_DICT)
+    {
+	clear_tv(&provider_tv);
+	return FAIL;
+    }
+    else if (dict_get_tv(
+		provider_tv.vval.v_dict,
+		(char *)function,
+		&action_tv) == FAIL)
+    {
+	clear_tv(&provider_tv);
+	return FAIL;
+    }
+    else if (action_tv.v_type != VAR_DICT)
+    {
+	clear_tv(&provider_tv);
+	clear_tv(&action_tv);
+	return FAIL;
+    }
+    else if (dict_get_tv(action_tv.vval.v_dict, (char *)reg, &func_tv) == FAIL)
+    {
+	clear_tv(&provider_tv);
+	clear_tv(&action_tv);
+	return FAIL;
+    }
+    else if ((cb = get_callback(&func_tv)).cb_name == NULL)
+    {
+	clear_tv(&provider_tv);
+	clear_tv(&action_tv);
+	clear_tv(&func_tv);
+	return FAIL;
+    }
+    clear_tv(&provider_tv);
+    clear_tv(&action_tv);
+
+    // func_tv owns the function name, so we must make a copy for the callback
+    set_callback(callback, &cb);
+    free_callback(&cb);
+    clear_tv(&func_tv);
+    return OK;
+}
+
+    static void
+clip_provider_copy(char_u *reg, char_u *provider)
+{
+    callback_T	callback;
+    typval_T	rettv;
+    typval_T	argvars[4];
+    yankreg_T	*y_ptr;
+    char_u	type[2 + NUMBUFLEN] = {0};
+    list_T	*list = NULL;
+
+    if (clip_provider_get_callback(
+		reg,
+		provider,
+		(char_u *)"copy",
+		&callback) == FAIL)
+	return;
+
+    // Convert register type into a string
+    if (*reg == '+')
+	y_ptr = get_y_register(REAL_PLUS_REGISTER);
+    else
+	y_ptr = get_y_register(STAR_REGISTER);
+
+    switch (y_ptr->y_type)
+    {
+	case MCHAR:
+	    type[0] = 'v';
+	    break;
+	case MLINE:
+	    type[0] = 'V';
+	    break;
+	case MBLOCK:
+	    sprintf((char *)type, "%c%d", Ctrl_V, y_ptr->y_width + 1);
+	    break;
+	default:
+	    type[0] = 0;
+	    break;
+    }
+
+    argvars[0].v_type = VAR_STRING;
+    argvars[0].vval.v_string = reg;
+
+    argvars[1].v_type = VAR_STRING;
+    argvars[1].vval.v_string = type;
+
+    // Get register contents by creating a list of lines
+    list = list_alloc();
+
+    if (list == NULL)
+    {
+	free_callback(&callback);
+	return;
+    }
+
+    for (int i = 0; i < y_ptr->y_size; i++)
+	if (list_append_string(list, y_ptr->y_array[i].string, -1) == FAIL)
+	{
+	    free_callback(&callback);
+	    list_unref(list);
+	    return;
+	}
+
+    list->lv_refcount++;
+
+    argvars[2].v_type = VAR_LIST;
+    argvars[2].v_lock = VAR_FIXED;
+    argvars[2].vval.v_list = list;
+
+    argvars[3].v_type = VAR_UNKNOWN;
+
+    textlock++;
+    call_callback(&callback, -1, &rettv, 3, argvars);
+    clear_tv(&rettv);
+    textlock--;
+
+    free_callback(&callback);
+    list_unref(list);
+}
+
+    static void
+clip_provider_paste(char_u *reg, char_u *provider)
+{
+    callback_T	callback;
+    typval_T	argvars[2];
+    typval_T	rettv;
+    int		ret;
+    char_u	*reg_type;
+    list_T	*lines;
+
+    if (clip_provider_get_callback(
+		reg,
+		provider,
+		(char_u *)"paste",
+		&callback) == FAIL)
+	return;
+
+    argvars[0].v_type = VAR_STRING;
+    argvars[0].vval.v_string = reg;
+
+    argvars[1].v_type = VAR_UNKNOWN;
+
+    textlock++;
+    ret = call_callback(&callback, -1, &rettv, 1, argvars);
+    textlock--;
+
+    if (ret == FAIL)
+	goto exit;
+    else if (rettv.v_type == VAR_TUPLE
+	    && TUPLE_LEN(rettv.vval.v_tuple) == 2
+	    && TUPLE_ITEM(rettv.vval.v_tuple, 0)->v_type == VAR_STRING
+	    && TUPLE_ITEM(rettv.vval.v_tuple, 1)->v_type == VAR_LIST)
+    {
+	reg_type = TUPLE_ITEM(rettv.vval.v_tuple, 0)->vval.v_string;
+	lines = TUPLE_ITEM(rettv.vval.v_tuple, 1)->vval.v_list;
+    }
+    else if (rettv.v_type == VAR_LIST
+	    && rettv.vval.v_list->lv_len == 2
+	    && rettv.vval.v_list->lv_first->li_tv.v_type == VAR_STRING
+	    && rettv.vval.v_list->lv_first->li_next->li_tv.v_type == VAR_LIST)
+    {
+	reg_type = rettv.vval.v_list->lv_first->li_tv.vval.v_string;
+	lines = rettv.vval.v_list->lv_first->li_next->li_tv.vval.v_list;
+    }
+    else
+	goto exit;
+
+    {
+	char_u		yank_type = MAUTO;
+	long		block_len = -1;
+	yankreg_T	*y_ptr, *cur_y_ptr;
+	char_u		**lstval;
+	char_u		**allocval;
+	char_u		buf[NUMBUFLEN];
+	char_u		**curval;
+	char_u		**curallocval;
+	char_u		*strval;
+	listitem_T	*li;
+	int		len;
+
+	// If the list is NULL handle like an empty list.
+	len = lines == NULL ? 0 : lines->lv_len;
+
+	// First half: use for pointers to result lines; second half: use for
+	// pointers to allocated copies.
+	lstval = ALLOC_MULT(char_u *, (len + 1) * 2);
+	if (lstval == NULL)
+	    goto exit;
+	curval = lstval;
+	allocval = lstval + len + 2;
+	curallocval = allocval;
+
+	if (lines != NULL)
+	{
+	    CHECK_LIST_MATERIALIZE(lines);
+	    FOR_ALL_LIST_ITEMS(lines, li)
+	    {
+		strval = tv_get_string_buf_chk(&li->li_tv, buf);
+		if (strval == NULL)
+		    goto free_lstval;
+		if (strval == buf)
+		{
+		    // Need to make a copy, next tv_get_string_buf_chk() will
+		    // overwrite the string.
+		    strval = vim_strsave(buf);
+		    if (strval == NULL)
+			goto free_lstval;
+		    *curallocval++ = strval;
+		}
+		*curval++ = strval;
+	    }
+	}
+	*curval++ = NULL;
+
+	if (*reg_type != NUL && (STRLEN(reg_type) <= 0
+		|| get_yank_type(&reg_type, &yank_type, &block_len) == FAIL))
+	{
+	    emsg(e_invalid_argument);
+	    goto free_lstval;
+	}
+
+	if (*reg == '+')
+	    y_ptr = get_y_register(REAL_PLUS_REGISTER);
+	else
+	    y_ptr = get_y_register(STAR_REGISTER);
+
+	// Free previous register contents
+	cur_y_ptr = get_y_current();
+	set_y_current(y_ptr);
+
+	free_yank_all();
+	get_y_current()->y_size = 0;
+
+	set_y_current(cur_y_ptr);
+
+	str_to_reg(y_ptr,
+		yank_type,
+		(char_u *)lstval,
+		-1,
+		block_len,
+		TRUE);
+
+free_lstval:
+	while (curallocval > allocval)
+	    vim_free(*--curallocval);
+	vim_free(lstval);
+    }
+
+exit:
+    free_callback(&callback);
+    clear_tv(&rettv);
+}
+
+// Used to stop calling the provider callback every time there is an update.
+// This prevents unnecessary calls when accessing the provider often in an
+// interval.
+//
+// If -1 then allow provider callback to be called then set to one. Default
+// value (is allowed) is -2.
+static int star_pause_count = -2, plus_pause_count = -2;
+
+    void
+call_clip_provider_request(int reg)
+{
+    if (clipmethod != CLIPMETHOD_PROVIDER)
+	return;
+
+    if (reg == '+' && plus_pause_count < 0)
+    {
+	if (plus_pause_count == -1)
+	    plus_pause_count = 1;
+	clip_provider_paste((char_u *)"+", clip_provider);
+    }
+    else if (reg == '*' && star_pause_count < 0)
+    {
+	if (star_pause_count == -1)
+	    star_pause_count = 1;
+	clip_provider_paste((char_u *)"*", clip_provider);
+    }
+    else
+	return;
+}
+
+    void
+call_clip_provider_set(int reg)
+{
+    if (clipmethod != CLIPMETHOD_PROVIDER)
+	return;
+
+    if (reg == '+' && plus_pause_count < 0)
+    {
+	if (plus_pause_count == -1)
+	    plus_pause_count = 1;
+	clip_provider_copy((char_u *)"+", clip_provider);
+    }
+    else if (reg == '*' && star_pause_count < 0)
+    {
+	if (star_pause_count == -1)
+	    star_pause_count = 1;
+	clip_provider_copy((char_u *)"*", clip_provider);
+    }
+}
+
+/*
+ * Makes it so that the next provider call is only done once any calls after are
+ * ignored, until dec_clip_provider is called the same number of times after
+ * again. Note that this is per clipboard register ("+", "*")
+ */
+    void
+inc_clip_provider(void)
+{
+    plus_pause_count = (plus_pause_count == -2
+	|| plus_pause_count == -1) ? -1 : plus_pause_count + 1;
+    star_pause_count = (star_pause_count == -2
+	|| star_pause_count == -1) ? -1 : star_pause_count + 1;
+}
+
+    void
+dec_clip_provider(void)
+{
+    if (plus_pause_count != -2)
+	plus_pause_count = plus_pause_count == -1 ? -1 : plus_pause_count - 1;
+    if (star_pause_count != -2)
+	star_pause_count = star_pause_count == -1 ? -1 : star_pause_count - 1;
+
+    if (plus_pause_count == 0 || plus_pause_count == -1)
+	plus_pause_count = -2;
+    if (star_pause_count == 0 || star_pause_count == -1)
+	star_pause_count = -2;
+}
+
+#endif // FEAT_CLIPBOARD_PROVIDER
